require 'minitest/autorun'
require 'pepito/handlers/room'

class TestPepitoHandlersHelp < MiniTest::Test
  Struct.new('Robot', :name, :handlers)
  Struct.new('Route', :help, :command)
  Struct.new('Handler', :chat_routes)

  def setup
    @mock_robot = Struct::Robot.new('pepito', [])
    @handler = Pepito::Handlers::Help.new(@mock_robot, {})
    @handler.run
  end

  def test_has_chat_routes
    chat_routes = ['help']
    @handler.chat_routes.each do |route|
      chat_routes.entries.each do |r|
        chat_routes.delete(r) if route.pattern.match(r)
      end
    end
    assert_empty chat_routes
  end

  def test_chat_help
    route_1 = Struct::Route.new('route1 help', false)
    route_2 = Struct::Route.new('route2 help', true)

    handler1 = Struct::Handler.new([route_1, route_2])
    handler2 = Struct::Handler.new([route_1, route_2])

    @mock_robot.handlers = { h1: handler1, h2: handler2 }

    chat_help = @handler.chat_help(nil, nil)
    expected = ['route1 help', 'pepito route2 help', 'route1 help', 'pepito route2 help']
    assert chat_help, expected
  end
end
