require 'minitest/autorun'
require 'pepito/adapter'
require 'pepito/errors/required_methods_error'

class StubAdapter < Pepito::Adapter
  class << self
    attr_accessor :configs
  end

  Pepito::Adapter::REQUIRED_METHODS.each do |arg|
    send :define_method, arg do
      nil
    end
  end
end

class TestPepitoAdapter < MiniTest::Test
  def test_raise_missing_methods
    StubAdapter.configs = [{}]
    e = assert_raises Pepito::Errors::RequiredMethodsError do
      Pepito::Adapter.new(nil, {})
    end

    assert_equal e.missing_methods, Pepito::Adapter::REQUIRED_METHODS
  end

  def test_no_missing_methods
    StubAdapter.configs = [{}]
    StubAdapter.new(nil, {})
    pass
  end

  def test_optional_config
    StubAdapter.configs = [{ name: 'optional', required: false }]
    StubAdapter.new(nil, {})
    pass
  end

  def test_required_config
    StubAdapter.configs = [{ name: 'required', required: true }]
    StubAdapter.new(nil, 'required' => 'config_value')
    pass
  end

  def test_missing_required_config
    StubAdapter.configs = [{ name: 'required', required: true }]
    e = assert_raises Pepito::Errors::MissingConfigurationValueError do
      StubAdapter.new(nil, {})
    end

    assert_equal e.configuration_values, ['required']
  end

  def test_nil_required_config
    StubAdapter.configs = [{ name: 'required', required: true }]
    e = assert_raises Pepito::Errors::MissingConfigurationValueError do
      StubAdapter.new(nil, 'required' => nil)
    end

    assert_equal e.configuration_values, ['required']
  end

  def test_empty_required_config
    StubAdapter.configs = [{ name: 'required', required: true }]
    e = assert_raises Pepito::Errors::MissingConfigurationValueError do
      StubAdapter.new(nil, 'required' => '')
    end

    assert_equal e.configuration_values, ['required']
  end
end
