require 'minitest/autorun'
require 'pepito/message'

class TestPepitoMessage < MiniTest::Test
  def test_command_false
    message = create_message('pepito', 'message')
    assert_equal message.command?, false
  end

  def test_command_false_if_name_empty
    message = create_message('', 'message')
    assert_equal message.command?, false
  end

  def test_command_false_if_name_empty_and_space_in_message
    message = create_message('', ' message')
    assert_equal message.command?, false
  end

  def test_command_true
    message = create_message('pepito', 'pepito message')
    assert_equal message.command?, true
    assert_equal message.command, 'message'
  end

  def test_route_match_true_if_command_false_and_route_command_false
    message = create_message('pepito', 'message')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, false)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), true
  end

  def test_route_match_false_if_command_false_and_route_command_false
    message = create_message('pepito', 'message asd')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, false)
    mock_route.expect(:command, false)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), false
  end

  def test_route_match_true_if_command_true_and_route_command_true
    message = create_message('pepito', 'pepito message')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, true)
    mock_route.expect(:command, true)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), true
  end

  def test_route_match_false_if_command_true_and_route_command_true
    message = create_message('pepito', 'pepito message asd')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, true)
    mock_route.expect(:command, true)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), false
  end

  def test_route_match_if_command_true_and_route_command_false
    message = create_message('pepito', 'pepito message')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, false)
    mock_route.expect(:command, false)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), false
  end

  def test_route_match_if_command_false_and_route_command_true
    message = create_message('pepito', 'message')

    mock_route = MiniTest::Mock.new
    mock_route.expect(:command, true)
    mock_route.expect(:command, true)
    mock_route.expect(:pattern, /^message$/)

    assert_equal message.match_route?(mock_route), false
  end

  private

  def create_message(name, body)
    mock_robot = MiniTest::Mock.new
    mock_robot.expect(:name, name)
    mock_robot.expect(:name, name)
    mock_robot.expect(:name, name)

    Pepito::Message.new(mock_robot, nil, body)
  end
end
